#!/usr/bin/env python
"""
Experiments with dawn, sunset and weather
"""

# CONSTANTS
TIMEOUT = 2 #seconds
DATE_FORMAT = "%a %b %d"
TIME_FORMAT = "%H:%M"
DATETIME_FORMAT = "%a %b %d %H:%M"
EMOJI = {
    "dawn": "🌄",
    "sunrise": "🌅",
    "noon": "🌞",
    "sunset": "🌇",
    "dusk": "🌃",
}
